#!/usr/bin/python
'''
  Deploys/creates (or updates) jenkins environment, neutron networks, etc.
  See README(s)!  This is not incorporated into jenkins jobs/tests.
'''
import logging
import os
import subprocess
import uuid
import yaml
from copy import deepcopy
from netaddr import IPNetwork
from neutronclient.neutron import client as ne_client
from novaclient import client as nv_client
from time import time
from common.osci_utils import (
    f_contains_str,
    floating_ip_is_available,
    get_networks,
    get_num_units,
    get_os_creds,
    has_floating_ip,
    read_yaml,
    recursive_dict_key_search,
    replace_line_with_lines,
    rnd_str,
    safe_mkdir,
    validate_config,
    write_yaml
)

# Set default logging level
logging.basicConfig(level=logging.DEBUG)

EV = 'environments'
DEPLOY_CONFIG_FILE = 'deployment/deploy-config.yaml'
JM_NAME_TEMPL = 'osci/{}'
SLAVE_NAME_TEMPLATES = {
    'slave': 'osci-slave/{}',  # jsv
    'lab': 'osci-lab/{}',      # jlb
    'task': 'osci-task/{}'     # jtk
}


def validate_network_config(conf):
    '''Confirm network config, verify cidr format using netaddr,
       check for sizing logic with regards to incrementation.
    '''
    ip_cidr = IPNetwork(conf['cidr'])
    ip_bits = int(ip_cidr.prefixlen)
    ret = ''

    if conf['cidr'] != str(ip_cidr.cidr):
        ret += ('Expected CIDR notation, ie. 10.100.0.0/16 but '
                'found: {}'.format(conf['cidr']))
    if ip_bits > 24:
        logging.warn('That may not be enough IP space ({}). '
                     'Proceeding anyway.'.format(ip_cidr))
    if int(conf['var-octet']) < 1 or int(conf['var-octet']) > 4:
        ret += ('Octet out of range: {}. '
                '1-4 are valid. '.format(conf['var-octet']))
    if conf['var-octet'] == '1':
        logging.warn('The first octet should not be targeted, unless '
                     'you have that public ip space.  Proceeding anyway.')
    if conf['var-octet'] == '2' and ip_bits < 16:
        ret += ('Can\'t split lower than /16 '
                'while incrementing 2nd octet!  ')
    if conf['var-octet'] == '3' and ip_bits < 24:
        ret += ('Can\'t split lower than /24 '
                'while incrementing 2nd octet!  ')
    if conf['var-octet'] == '4':
        ret += 'Can\'t use 4th octet.  Recommend 2nd.'

    logging.debug('ip network: {}'.format(ip_cidr.network))
    logging.debug('ip prefixlen: {}'.format(ip_cidr.prefixlen))
    return ret


def get_junits(conf_env, jsv_qty, jtk_qty, jlb_qty):
    '''Create a list of juju jenkins master and slave service
       units, predictively.
    '''
    units = []
    units.append(JM_NAME_TEMPL.format('0'))

    for _templ_qty_map in [(SLAVE_NAME_TEMPLATES['slave'], jsv_qty),
                           (SLAVE_NAME_TEMPLATES['task'], jtk_qty),
                           (SLAVE_NAME_TEMPLATES['lab'], jlb_qty)]:
        _templ = _templ_qty_map[0]
        _qty = _templ_qty_map[1]
        for unit in range(0, _qty):
            units.append(_templ.format(unit))

    return units


def get_enviros(conf_env, jsv_qty, jtk_qty, jlb_qty):
    '''Return a list of juju model names to create'''
    envs = []
    envs.append("{}-{}{}".format(conf_env['env-name-base'],
                                 conf_env['master-abbrv'],
                                 0))

    for _abbrv_qty_map in [(conf_env['slave-abbrv'], jsv_qty),
                           (conf_env['task-abbrv'], jtk_qty),
                           (conf_env['lab-abbrv'], jlb_qty)]:
        _abbrv = _abbrv_qty_map[0]
        _qty = _abbrv_qty_map[1]
        for unit in range(0, _qty):
            envs.append("{}-{}{}".format(conf_env['env-name-base'],
                                         _abbrv,
                                         str(unit).zfill(2)))
    return envs


def build_nets_and_subs(d_env_data, creds, conf):
    '''Build new neutron subnets and networks from dict data, and connect
       to neutron router.'''
    # Build new neutron networks, subnets, and connect them to the router
    neutron = ne_client.Client('2.0', **creds[0])

    conf_net = conf['osci']['network']
    conf_env = conf['osci']['enviro']

    base_name = conf_env['env-name-base']
    var_oct = conf_net['var-octet']
    router_name = conf_net['neutron-router']
    dns = conf_net['neutron-dns']

    for k, conf in d_env_data['enviros'].iteritems():
        net_str = conf['network']
        sub_name = "{}_{}_{}".format(base_name,
                                     net_str.split('.')[int(var_oct) - 1],
                                     'sub')
        net_name = "{}_{}_{}".format(base_name,
                                     net_str.split('.')[int(var_oct) - 1],
                                     'net')

        # Create neutron network
        networks = neutron.list_networks(name=net_name)
        if len(networks['networks']) == 0:
            logging.info('Creating network: %s',
                         net_name)
            network_msg = {
                'network': {
                    'name': net_name,
                    'shared': False
                }
            }
            neu_network = neutron.create_network(network_msg)['network']
        else:
            logging.info('Network %s already exists.', net_name)
            neu_network = networks['networks'][0]

        # Check if subnet name and IP space is free here
        subnets = neutron.list_subnets(name=sub_name)
        if len(subnets['subnets']) == 0:
            logging.info('Creating subnet: %s', sub_name)
            this_cidr = IPNetwork(net_str)
            alloc_start = this_cidr[2]
            alloc_end = this_cidr[-57]  # leave some static space available

            subnet_msg = {
                'subnet': {
                    'name': sub_name,
                    'network_id': neu_network['id'],
                    'enable_dhcp': True,
                    'cidr': str(this_cidr),
                    'ip_version': 4,
                    'dns_nameservers': dns.split(',')
                }
            }
            subnet_msg['subnet']['allocation_pools'] = [{
                'start': str(alloc_start),
                'end': str(alloc_end)
            }]
            logging.info(subnet_msg)
            neu_subnet = neutron.create_subnet(subnet_msg)['subnet']
        else:
            logging.info('Subnet %s already exists.', net_name)
            neu_subnet = subnets['subnets'][0]

        # Wire the network to the router
        routers = neutron.list_routers(name=router_name)
        if len(routers['routers']) == 0:
            raise ValueError('Unable to locate router')
        else:
            ports = neutron.list_ports(device_owner='network:router_interface',
                                       network_id=neu_network['id'])
            if len(ports['ports']) == 0:
                logging.info('Adding interface from %s to %s',
                             router_name, sub_name)
                neu_router = routers['routers'][0]
                neutron.add_interface_router(neu_router['id'],
                                             {'subnet_id': neu_subnet['id']})
            else:
                logging.info('Router already connected to subnet %s', sub_name)

        d_env_data['enviros'][k]['network-id'] = str(neu_network['id'])


def is_deployed(jma_num_units, jsv_num_units, jtk_num_units, jlb_num_units):
    '''Check juju status against expected number of jenkins
       master and slave units
    '''
    ret = True
    try:
        # Deployed, check unit quantities
        jm_stat = yaml.load(subprocess.check_output(
                            ['juju', 'status', 'osci']))
        jsv_stat = yaml.load(subprocess.check_output(
                            ['juju', 'status', 'osci-slave']))
        jtk_stat = yaml.load(subprocess.check_output(
                            ['juju', 'status', 'osci-task']))
        jlb_stat = yaml.load(subprocess.check_output(
                            ['juju', 'status', 'osci-lab']))

        jma_count = len(jm_stat['services']['osci']['units'])
        jsv_count = len(jsv_stat['services']['osci-slave']['units'])
        jtk_count = len(jtk_stat['services']['osci-task']['units'])
        jlb_count = len(jlb_stat['services']['osci-lab']['units'])

        if int(jma_count) != jma_num_units:
            ret = ('Juju model is up, but jenkins master'
                   'count is unexpected: {}/{}'.format(jma_count,
                                                       jma_num_units))
        if int(jsv_count) != jsv_num_units:
            ret = ('Juju model is up, but jenkins slave count is '
                   'unexpected: {}/{}'.format(jsv_count, jsv_num_units))
        if int(jtk_count) != jtk_num_units:
            ret = ('Juju model is up, but task slave count is '
                   'unexpected: {}/{}'.format(jtk_count, jtk_num_units))
        if int(jlb_count) != jlb_num_units:
            ret = ('Juju model is up, but lab slave count is '
                   'unexpected: {}/{}'.format(jlb_count, jlb_num_units))
    except (subprocess.CalledProcessError, KeyError):
        # Not deployed
        ret = False
    return ret


def make_post_file_xfer_script(data, script_file_abs):
    '''Create post-deployment file transfer script, which utilizes
       juju scp and juju ssh to copy files into the jenkins units,
       such as novarc and environments.yaml, and to set the permissions.
    '''
    cmds = []
    for d_file in data:
        # Separator
        cmds.append('echo -e "\\n----{}: {}----"'.format(d_file['unit'],
                                                         d_file['src-name']))

        # Copy some files to the unit
        juju_scp_cmd = 'juju scp {}/{} {}:{}/{}'.format(d_file['src-dir'],
                                                        d_file['src-name'],
                                                        d_file['unit'],
                                                        d_file['tmp-dir'],
                                                        d_file['dst-name'])
        cmds.append(juju_scp_cmd)

        # Modify files inside the unit
        cmd_bash = ''
        cmd_bash += "sudo su - jenkins -c 'mkdir -p {}' && ".format(
                    d_file['dst-dir'])
        cmd_bash += "sudo mv {}/{} {}/{} && ".format(d_file['tmp-dir'],
                                                     d_file['dst-name'],
                                                     d_file['dst-dir'],
                                                     d_file['dst-name'])
        cmd_bash += "sudo chown -R {} {} && ".format(
                    d_file['chown'],
                    d_file['dst-dir'])
        cmd_bash += "sudo chmod {} {}/{}".format(d_file['chmod'],
                                                 d_file['dst-dir'],
                                                 d_file['dst-name'])
        juju_ssh_cmd = 'juju ssh {} "{}"'.format(d_file['unit'], cmd_bash)
        cmds.append(juju_ssh_cmd)

    # Write file
    with open(script_file_abs, 'w') as out_file:
        out_file.write('{}\n'.format('#!/bin/bash -e'))
        for cmd in cmds:
            out_file.write('{}\n'.format(cmd))

    subprocess.check_call(['chmod',
                           '770', script_file_abs])

    return script_file_abs


def create_env_yamls_and_post_xfer_dict(data, conf, creds):
    '''Create juju environment yaml files for each master and slave,
       create and return absolute path of bash script which pushes
       files into the juju service units.
    '''
    f_template = read_yaml(conf['env-template'])
    f_reference = read_yaml(conf['env-reference'])
#    series = read_yaml('deployment/deploy-files/environment-series.yaml')
    apt_proxy = recursive_dict_key_search(f_reference, 'apt-http-proxy')
    jenk_home = '/var/lib/jenkins'
    u_home = '/home/ubuntu'
    _files = []

    for env_name, env_attr in data['enviros'].iteritems():
        d_tmp = deepcopy(f_template)
        env_name_dev = '{}-jdev'.format(env_name)
        env_name_prop = '{}-jprop'.format(env_name)

        # Set this slave's juju model name
        d_tmp[EV][env_name] = deepcopy(d_tmp[EV]['__ENVIRO__'])
        d_tmp['default'] = env_name
        del d_tmp[EV]['__ENVIRO__']

        # Set juju model openstack parameters
        d_tmp[EV][env_name]['admin-secret'] = rnd_str(length=32)
        d_tmp[EV][env_name]['auth-url'] = creds[0]['auth_url']
        d_tmp[EV][env_name]['control-bucket'] = str(uuid.uuid1())
        d_tmp[EV][env_name]['password'] = creds[0]['password']
        d_tmp[EV][env_name]['tenant-name'] = creds[0]['tenant_name']
        d_tmp[EV][env_name]['username'] = creds[0]['username']

        if 'network-id' in env_attr:
            d_tmp[EV][env_name]['network'] = env_attr['network-id']

        if apt_proxy is not None:
            d_tmp[EV][env_name]['apt-http-proxy'] = apt_proxy
        else:
            del d_tmp[EV][env_name]['apt-http-proxy']

        # Create juju-devel model yaml for environments.yaml
        d_tmp[EV][env_name_dev] = deepcopy(d_tmp[EV][env_name])
        d_tmp[EV][env_name_dev]['agent-stream'] = 'devel'

        # Create juju-proposed model yaml for environments.yaml
        d_tmp[EV][env_name_prop] = deepcopy(d_tmp[EV][env_name])
        d_tmp[EV][env_name_prop]['agent-stream'] = 'proposed'

        # Create juju model yaml for environments.yaml
        safe_mkdir(conf['out-dir'])
        env_filename = 'environments-{}.yaml'.format(env_name)
        env_file_abs = '{}/{}'.format(conf['out-dir'], env_filename)
        write_yaml(d_tmp, env_file_abs)
        logging.debug('created environment file: {}'.format(env_file_abs))

        # Add to post-deployment file xfer dict
        _files.append(
            {'src-name': env_filename,
             'dst-name': 'environments.yaml',
             'src-dir': conf['out-dir'],
             'tmp-dir': u_home,
             'dst-dir': '{}/.juju'.format(jenk_home),
             'chown': 'jenkins:jenkins',
             'chmod': '600',
             'unit': env_attr['unit']
             })
        _files.append(
            {'src-name': 'novarc',
             'dst-name': 'novarc',
             'src-dir': os.getenv("HOME"),
             'tmp-dir': u_home,
             'dst-dir': jenk_home,
             'chown': 'jenkins:jenkins',
             'chmod': '600',
             'unit': env_attr['unit']
             })

    # also set master's cli.conf
    _files.append(
        {'src-name': 'cli.conf',
         'dst-name': 'cli.conf',
         'src-dir': 'deployment/deploy-files',
         'tmp-dir': u_home,
         'dst-dir': '{}/.jenkins'.format(jenk_home),
         'chown': 'jenkins:jenkins',
         'chmod': '600',
         'unit': 'osci/0'
         })

    return _files


def parse_jenkins_config_xml(conf):
    '''Pull config.xml from from the jenkins master, replace or
       inject lines of config, return list of dict of files/attributes
       for handling by the post-xfer routine.
    '''

    # Get config.xml from jenkins master
    time_stamp = str(time()).replace('.', '-')
    file_name = 'config.xml.{}'.format(time_stamp)

    subprocess.check_call(['juju', 'ssh', 'osci/0',
                           'sudo cp /var/lib/jenkins/config.xml '
                           '/home/ubuntu/{}'.format(file_name)])
    subprocess.check_call(['juju', 'ssh', 'osci/0',
                           'sudo chown ubuntu:ubuntu '
                           '/home/ubuntu/{}'.format(file_name)])
    juju_scp_cmd = ('juju scp {}:{} {}'.format('osci/0',
                    '/home/ubuntu/{}'.format(file_name),
                    conf['out-dir']))
    subprocess.check_call(juju_scp_cmd.split())

    loc_xml = '{}/{}'.format(conf['out-dir'], file_name)
    do_copy = False
    _files = []

    operations = []
    operations.extend(
        [{'replace': 'FullControlOnceLoggedInAuthorizationStrategy',
          'with': 'deployment/deploy-files/config.xml.authstrat'
          },
         {'replace': '<globalNodeProperties/>',
          'with': 'deployment/deploy-files/config.xml.global'
          }])

    if not f_contains_str(the_file=loc_xml, str_find='<listView>'):
        operations.append(
            {'replace': '</views>',
             'with': 'deployment/deploy-files/config.xml.views'
             })

    for _op in operations:
        logging.info('op: {}:{}:{}'.format(loc_xml,
                                           _op['replace'],
                                           _op['with']))

        with open(_op['with']) as inject_file:
            inject_lines = inject_file.readlines()
            ret = replace_line_with_lines(file_name=loc_xml,
                                          str_find=_op['replace'],
                                          with_lines=inject_lines)
            if ret:
                # No need, not modified.
                logging.info(ret)
            else:
                do_copy = True

    if do_copy:
        # Add config.xml to the list to copy back to jenkins master
        logging.info('do_copy True')
        _files.append(
            {'src-name': file_name,
             'dst-name': 'config.xml',
             'src-dir': conf['out-dir'],
             'tmp-dir': '/home/ubuntu',
             'dst-dir': '/var/lib/jenkins',
             'chown': 'jenkins:jenkins',
             'chmod': '644',
             'unit': 'osci/0'
             })
    return _files


def jenkins_other_xml():
    '''Build files list for additional jenkins xml files
       to push into the master
    '''
    _files = []
    _files.append(
        {'src-name': 'hudson.tasks.Mailer.xml',
         'dst-name': 'hudson.tasks.Mailer.xml',
         'src-dir': 'deployment/deploy-files',
         'tmp-dir': '/home/ubuntu',
         'dst-dir': '/var/lib/jenkins',
         'chown': 'jenkins:jenkins',
         'chmod': '644',
         'unit': 'osci/0'
         })
    _files.append(
        {'src-name': 'hudson.plugins.build_publisher.BuildPublisher.xml',
         'dst-name': 'hudson.plugins.build_publisher.BuildPublisher.xml',
         'src-dir': 'deployment/deploy-files',
         'tmp-dir': '/home/ubuntu',
         'dst-dir': '/var/lib/jenkins',
         'chown': 'jenkins:jenkins',
         'chmod': '644',
         'unit': 'osci/0'
         })
    return _files


def float_jenkins_master(creds, conf):
    '''Associate a floating IP with the jenkins master instance (create
       floating IP if necessary).
    '''
    conf_net = conf['osci']['network']

    neutron = ne_client.Client('2.0', **creds[0])
    nova = nv_client.Client('2', **creds[1])
    jm_stat = yaml.load(subprocess.check_output(['juju', 'status', 'osci']))
    jm_id = jm_stat['machines'].keys()[0]
    jm_instance_id = jm_stat['machines'][jm_id]['instance-id']
    jm_instance = nova.servers.get(server=jm_instance_id)
    jm_addresses = jm_instance.addresses[conf_net['default-network']]
    ext_net_id = (neutron.list_networks(name=conf_net['ext-net'])
                  ['networks'][0]['id'])

    ret = has_floating_ip(jm_addresses, conf_net['fip-str'])
    if ret:
        logging.info('Instance already has a floating ip: {}'.format(ret))
    else:
        if floating_ip_is_available(neutron):
            logging.info('Floating ip exists in tenant, associating it.')
            jm_float_ip = (neutron.list_floatingips(status='DOWN')
                           ['floatingips'][0]['floating_ip_address'])
        else:
            logging.info('No free floating ip(s) in tenant, creating one.')
            fip_msg = {
                'floatingip': {
                    'floating_network_id': ext_net_id
                }
            }
            try:
                jm_float_ip = (neutron.create_floatingip(fip_msg)['floatingip']
                               ['floating_ip_address'])

            except Exception:
                raise ValueError('Tenant could not create a floating IP')

        # Associate floating ip with instance
        try:
            jm_instance.add_floating_ip(address=jm_float_ip)
        except Exception:
            raise ValueError('Failed to associate floating ip '
                             '({}) to instance {}.'.format(jm_float_ip,
                                                           jm_instance_id))

        # Confirm
        jm_instance = nova.servers.get(server=jm_instance_id)
        jm_addresses = jm_instance.addresses[conf_net['default-network']]
        ret = has_floating_ip(jm_addresses, conf_net['fip-str'])
        if ret:
            logging.info('Instance has floating ip: {}'.format(ret))


def main():
    '''Read and validate configs, build networks, deploy bundle,
       perform some post-deploy config and file transfers.
    '''
    # Read config from yaml
    conf = read_yaml(DEPLOY_CONFIG_FILE)
    creds = get_os_creds()

    # Basic non-null general parameter validation
    expected = ['env-name-base', 'master-abbrv', 'slave-abbrv', 'split',
                'env-template', 'out-dir', 'deployer-bundle']
    ret = validate_config(conf['osci'], expected)
    if ret:
        logging.debug('conf: {}'.format(conf['osci']))
        raise ValueError('Config data missing. {}'.format(ret))

    # Break out config parameters
    net_split = conf['osci']['network']['split']
    conf_net = conf['osci']['network']
    conf_env = conf['osci']['enviro']

    # Inspect deployer bundle yaml for needed info & validate
    deployer_bundle = conf['osci']['deployer-bundle']
    d_dep_bundle = read_yaml(deployer_bundle)
    jma_num_units = get_num_units(d_dep_bundle,
                                  d_dep_bundle.keys()[0],
                                  'osci')
    jsv_num_units = get_num_units(d_dep_bundle,
                                  d_dep_bundle.keys()[0],
                                  'osci-slave')
    jtk_num_units = get_num_units(d_dep_bundle,
                                  d_dep_bundle.keys()[0],
                                  'osci-task')
    jlb_num_units = get_num_units(d_dep_bundle,
                                  d_dep_bundle.keys()[0],
                                  'osci-lab')
    logging.debug('jma: {}\njsv: {}\njtk: {}\njlb: {}'.format(
        jma_num_units, jsv_num_units, jtk_num_units, jlb_num_units))

    # Validate bundle unit counts before deploying
    if jma_num_units != 1:
        raise ValueError('Expected exactly 1 jenkins master, '
                         'found: {}'.format(jma_num_units))
    if jsv_num_units < 1:
        raise ValueError('Expected at least 1 jenkins slave, '
                         'found: {}'.format(jsv_num_units))
    if jtk_num_units < 1:
        raise ValueError('Expected at least 1 task slave, '
                         'found: {}'.format(jtk_num_units))
    if jlb_num_units < 1:
        raise ValueError('Expected at least 1 lab slave, '
                         'found: {}'.format(jlb_num_units))

    # Deploy if not already deployed
    ret = is_deployed(jma_num_units, jsv_num_units,
                      jtk_num_units, jlb_num_units)
    if ret is not True and ret is not False:
        raise ValueError(ret)
    elif ret is True:
        logging.info('Model is already deployed, moving on.')
    elif ret is False:
        logging.info('Model is not deployed, deploying...')
        subprocess.check_call(['juju-deployer',
                               '-v', '-d', '-B',
                               '-c', deployer_bundle])
        # Confirm new deployment
        ret = is_deployed(jma_num_units, jsv_num_units,
                          jtk_num_units, jlb_num_units)
        if ret is not True:
            raise ValueError(ret)

    # Predictively construct new juju environment and unit names lists
    junits = get_junits(conf_env=conf_env,
                        jsv_qty=jsv_num_units,
                        jtk_qty=jtk_num_units,
                        jlb_qty=jlb_num_units)

    enviros = get_enviros(conf_env=conf_env,
                          jsv_qty=jsv_num_units,
                          jtk_qty=jtk_num_units,
                          jlb_qty=jlb_num_units)

    # Network separation
    if net_split is False:
        logging.info('No network separation.')
        nets = [None] * len(junits)
    elif net_split is True:
        logging.info('Using layer 2 network separation.')

        # Basic non-null network parameter validation
        expected = ['cidr', 'var-octet', 'neutron-router',
                    'neutron-net', 'neutron-sub', 'neutron-dns',
                    'ext-net', 'fip-str']
        ret = validate_config(conf_net, expected)
        if ret:
            raise ValueError('Network config data missing. {}'.format(ret))

        # Validate IP as CIDR;  Validate network size and incrementating logic
        ret = validate_network_config(conf=conf_net)
        if ret:
            raise ValueError('IP, network or logic issue(s): {}'.format(ret))

        # Construct new network config data list
        ip_cidr = IPNetwork(conf_net['cidr'])
        total_units = (jma_num_units + jsv_num_units +
                       jtk_num_units + jlb_num_units)
        nets = get_networks(network=str(ip_cidr.network),
                            bits=int(ip_cidr.prefixlen),
                            octet=int(conf_net['var-octet']),
                            quantity=total_units)

    # Ensure lists are the same length & smash into a dictionary
    check_len = [len(enviros), len(nets), len(junits)]
    if check_len != [check_len[0]] * 3:
        logging.debug('enviros ({}): {}'.format(len(enviros), enviros))
        logging.debug('nets ({}): {}'.format(len(nets), nets))
        logging.debug('junits ({}): {}'.format(len(junits), junits))
        raise ValueError('Model, network and juju unit counts should be equal')
    d_env_data = {'enviros': {}}
    for idx, enviro in enumerate(enviros):
        d_env_data['enviros'][enviro] = {
            'network': nets[idx],
            'unit': junits[idx]
        }

    # Add neutron networks, subnets and router connections, if called for
    if net_split is True:
        build_nets_and_subs(d_env_data=d_env_data,
                            creds=creds,
                            conf=conf)

    # Give jenkins master a floating IP if it doesn't already have one
    float_jenkins_master(creds=creds, conf=conf)

    # Create environment yaml file for each jenkins master & slave unit,
    # return a dict of files to copy into those units.
    xfer_files = create_env_yamls_and_post_xfer_dict(data=d_env_data,
                                                     conf=conf_env,
                                                     creds=creds)

    # Grab config.xml from jenkins master, replace certain sections
    ret = parse_jenkins_config_xml(conf_env)
    if ret:
        xfer_files.extend(ret)

    # Push in build publisher, mailer, other jenkins config xmls
    ret = jenkins_other_xml()
    if ret:
        xfer_files.extend(ret)

    # Write d_env_data out to yaml for reference
    logging.debug('New model and network predictions: {}'.format(
                  yaml.dump(d_env_data, default_flow_style=False)))
    safe_mkdir(conf_env['out-dir'])
    write_yaml(d_env_data, '{}/env_data.yaml'.format(conf_env['out-dir']))

    # Build and execute post-deployment file xfer script
    post_file = make_post_file_xfer_script(
        data=xfer_files, script_file_abs='{}/{}'.format(
            conf_env['out-dir'], 'post-xfer.sh'))

    if os.path.isfile(post_file):
        subprocess.check_call([post_file])
    else:
        raise ValueError('Failed to create post-deployment file xfer script.')


if __name__ == '__main__':
    main()
