#!/bin/bash -e
#
#  Common functions for UOSCI jobs
#
#  NOTE(beisner):  Many/most of these funcs rely on environment variables being
#  set by the build parameters or the calling script(s).

. ${OSCI_ROOT}/job-parts/osci_tools_common.sh
. ${OSCI_ROOT}/job-parts/osci_openstack_common.sh
. ${OSCI_ROOT}/job-parts/osci_jujudevel_common.sh
. ${OSCI_ROOT}/job-parts/osci_mojo_common.sh
. ${OSCI_ROOT}/job-parts/osci_osi_common.sh
. ${OSCI_ROOT}/job-parts/osci_apt_common.sh
. ${OSCI_ROOT}/job-parts/osci_gerrit_common.sh
. ${OSCI_ROOT}/job-parts/osci_layerci_common.sh
. ${OSCI_ROOT}/job-parts/osci_juju_common.sh


function f_juju_wait() {
# use juju-wait to block until hooks and relations have settled
  f_line "Waiting for deployment to settle (juju-wait)..."
  [[ -z "$JUJU_WAIT_TIMEOUT" ]] && export JUJU_WAIT_TIMEOUT=1800
  timeout $JUJU_WAIT_TIMEOUT $JUJU_WAIT_ROOT/juju-wait -v 2>&1 | tee $WORKSPACE/juju-wait.txt
  if [[ "${PIPESTATUS[0]}" != "0" ]]; then
    echo " ! juju-wait failed."
    touch $WORKSPACE/fyi-${test:0:4}.${test}-juju-wait.failed
  else
    echo " . juju-wait OK."
  fi
}


function f_juju_wait_on_workload() {
# use juju-wait to block until hooks and relations have settled
# also wait for workload status on all units
  f_line "Waiting for deployment to settle (juju-wait)..."
  [[ -z "$JUJU_WAIT_TIMEOUT" ]] && export JUJU_WAIT_TIMEOUT=1800
  timeout $JUJU_WAIT_TIMEOUT $JUJU_WAIT_ROOT/juju-wait -vw 2>&1 | tee $WORKSPACE/juju-wait.txt
  if [[ "${PIPESTATUS[0]}" != "0" ]]; then
    echo " ! juju-wait failed."
    touch $WORKSPACE/fyi-${test:0:4}.${test}-juju-wait.failed
  else
    echo " . juju-wait OK."
  fi
}


function f_juju1_get_model_name() {
# Determine Juju 1 model name to run with
  juju_enviro_file="${HOME}/.juju/environments.yaml"
  if [[ "$MOJO_SPEC" != *metal* ]]; then
    # Not bare metal
    case "$JUJU_SOURCE" in
      $JUJU_STABLE)
        echo " . Juju stable"
        JENV_NAME="$(grep '^default:' ${juju_enviro_file} | head -n1 | cut -f2 -d ':' | xargs)"
        ;;
      $JUJU_PROPOSED)
        echo " . Juju proposed"
        JENV_NAME="$(grep '.jprop:$' ${juju_enviro_file} | head -n1 | cut -f1 -d ':' | xargs)"
        ;;
      $JUJU_DEVEL)
        echo " . Juju devel"
        JENV_NAME="$(grep '.jdev:$' ${juju_enviro_file} | head -n1 | cut -f1 -d ':' | xargs)"
        ;;
      *)
        echo " ! Unknown JUJU_SOURCE value while determining jenv name."
        echo "JUJU_SOURCE: $JUJU_SOURCE"
        exit 1
        ;;
    esac
  else
    # Bare metal
    echo " . Handling juju environment switcher for bare metal..."
    case "$JUJU_SOURCE" in
      $JUJU_STABLE)
        echo " . Juju stable"
        JENV_NAME="$(grep "maas-${MOJO_SERIES}:$" ${juju_enviro_file} | head -n1 | cut -f1 -d ':' | xargs)"
        ;;
      $JUJU_PROPOSED)
        echo " . Juju proposed"
        JENV_NAME="$(grep .jprop:$ ${juju_enviro_file} | grep "maas-${MOJO_SERIES}" | head -n1 | cut -f1 -d ':' | xargs)"
        ;;
      $JUJU_DEVEL)
        echo " . Juju devel"
        JENV_NAME="$(grep .jdev:$ ${juju_enviro_file} | grep "maas-${MOJO_SERIES}" | head -n1 | cut -f1 -d ':' | xargs)"
        ;;
      *)
        echo " ! Unknown JUJU_SOURCE value while determining jenv name."
        echo "JUJU_SOURCE: $JUJU_SOURCE"
        exit 1
        ;;
    esac
  fi

  # Validate as non-empty
  if [[ -z "$JENV_NAME" ]]; then
    echo " ! Could not determine an appropriate JENV_NAME."
        echo "JUJU_SOURCE: $JUJU_SOURCE"
    exit 1
  fi
  export JENV_NAME
  echo " . JUJU_SOURCE: $JUJU_SOURCE"
  echo " . JENV_NAME: $JENV_NAME"
}


function f_juju_status_health_check() {
# Iterate through all juju machines and all juju units/services
# Check agent stat and connectivity to each.

  [[ -n "$1" ]] && PARAM1="$1" || PARAM1=""
  f_line "Checking juju stat health: $PARAM1"

  # Run jshc from a venv as a work-around to:
  #   https://bugs.launchpad.net/ubuntu/+source/paramiko/+bug/1665565
  #   https://bugs.launchpad.net/charm-test-infra/+bug/1665598
  # ${OSCI_ROOT}/tools/juju_status_health_check.py --Xb 2>&1 | tee $WORKSPACE/juju-stat-health-check${PARAM1}.txt; ret_code=${PIPESTATUS[0]}
  cd ${OSCI_ROOT}/tools
  deactivate ||:
  tox -re jshc | tee $WORKSPACE/juju-stat-health-check${PARAM1}.txt; ret_code=${PIPESTATUS[0]}
  echo "Returned:  ${ret_code}"
  case "$ret_code" in
    0)
      echo " . FYI - Deployment health (${PARAM1}) OK.";
      ;;
    *)
      # Suppress error if running in the collection phase
      if [[ "$PARAM1" != "-collect" ]]; then
        echo " ! FYI - Deployment health (${PARAM1}) BAD.";
        touch $WORKSPACE/fyi.juju-stat-health-check${PARAM1}.failed;
      fi
      ;;
   esac
}


function f_node_sys_info() {
  f_line "General node system info"
  uptime && echo
  df -h / && echo
  df -h /mnt && echo
  free -m && echo
}


function f_ssh_known_hosts_clean() {
# Ensure clean slate with known hosts
  echo "Overwriting with a clean known_hosts file on ${NODE_NAME}..."
  cat $OSCI_ROOT/deployment/deploy-files/launchpad_host_keys > /var/lib/jenkins/.ssh/known_hosts
  cat $OSCI_ROOT/deployment/deploy-files/git_host_keys >> /var/lib/jenkins/.ssh/known_hosts
  cat $OSCI_ROOT/deployment/deploy-files/gerrit_host_keys >> /var/lib/jenkins/.ssh/known_hosts
  cat $OSCI_ROOT/deployment/deploy-files/other_host_keys >> /var/lib/jenkins/.ssh/known_hosts
  chmod 600 /var/lib/jenkins/.ssh/known_hosts
  chown jenkins:jenkins /var/lib/jenkins/.ssh/known_hosts
}


function f_workspace_cleanup() {
# Purge all files from current workspace
  echo "Checking workspace: ${WORKSPACE} for ${JOB_NAME} on ${NODE_NAME}..."
  if [[ -d "$WORKSPACE" ]]; then
    if [[ -n "$(echo $WORKSPACE | grep '/var/lib/jenkins')" ]]; then
      echo " - Removing workspace items..."
      rm -rf $WORKSPACE/*
    else
      # Paranoid safety mechanism
      echo " ! WORKSPACE is not subdir of Jenkins home path, removed nothing!"
    fi
  else
    echo " ! WORKSPACE directory doesn't exist, removed nothing!"
  fi
}


function f_fresh_clean_start() {
# Generally run at the start of every build
# when no artifacts are copied into the workspace.
  f_workspace_cleanup
  f_ssh_known_hosts_clean
  f_refreshtools_jujuclean
}


function f_refreshtools_jujuclean() {
# Generally run at the start of every build
# when artificats are copied into the workspace from
# another project, otherwise generally as 2nd step.
  f_node_sys_info
  f_tools_refresh

  if [[ "${SKIP_JUJU_LXC^^}" != "TRUE" ]]; then
    ${OSCI_ROOT}/job-parts/juju1_bighammer_destroy.sh YES
    f_delete_neutron_ext_net_ports
    f_mojo_broom
    f_osi_broom
    f_juju_get_pkg_info tools-pkg-info.txt
  fi
}


function f_touch_try() {
# Create touchfile, indicating that we tried to run this test.
# Valid only in the LP workflow.
  if [[ -n "$MP_TRIGGER" ]]; then
    touch $WORKSPACE/try-${test:0:4}
  fi
}


function f_env_no_pass() {
# Dump environment variables, except for password vars
  env | egrep -v "PASSWORD|password|COOKIE|cookie|OS_|USER|user|PROXY|proxy" | \
    sort 2>&1 | tee $WORKSPACE/env.txt
}


function f_list_nova_instances() {
  # DEBUG: list all running instances
  f_novarc_undercloud
  if [[ "${JENV_NAME^^}" != *MAAS* ]]; then
    f_line "Listing nova instances to artifact"
    # Trap true, as instances may not exist
    nova list | grep "\-${JENV_NAME}\-machine" &> $WORKSPACE/nova-instances-$JENV_NAME.txt || :
    nova list &> $WORKSPACE/nova-instances-all.txt || :
  fi
}


function f_list_neutron_ports() {
  # DEBUG: list all tenant neutron ports
  f_novarc_undercloud
  if [[ "${JENV_NAME^^}" != *MAAS* ]]; then
    f_line "Listing neutron ports artifact"
    # Trap true, as ports may not exist
    neutron port-list &> $WORKSPACE/neutron-ports.txt || :
  fi
}


function f_delete_neutron_ext_net_ports() {
  # CLEANUP: delete neutron ext-net ports, commonly added by mojo or other
  # post-deployment configuration for the network node's 2nd nic.
  f_novarc_undercloud
  if [[ -z "${JENV_NAME}" ]]; then
    echo " ! Missing required env var JENV_NAME"
    exit 1
  fi
  if [[ ! -f "${UOSCI_BOT_CONTROL_ROOT}/tools/juju-sym-switch" ]]; then
    f_tool_refresh_bot_control
  fi
  if [[ "${JENV_NAME^^}" != *MAAS* ]]; then
    f_line "Deleting undercloud tenant's neutron ext-net ports if they exist"
    pwd
    cd ${UOSCI_BOT_CONTROL_ROOT}/tools
    deactivate ||:
    tox -re port-cleanup
    . .tox/port-cleanup/bin/activate
    ${UOSCI_BOT_CONTROL_ROOT}/tools/port-cleanup -pn "juju-${JENV_NAME}.*-machine.*ext-port" -ps "DOWN" --delete
    cd $WORKSPACE
    pwd
  fi
}


function f_delete_secgroups() {
  # CLEANUP: delete nova security groups which are often not deleted
  # by Juju destroy.
  #   ex:   juju-osci-sv01-31
  #   ref:  https://bugs.launchpad.net/juju-core/+bug/1333162
  #         https://bugs.launchpad.net/juju/+bug/1570216

  f_novarc_undercloud
  if [[ -z "${JENV_NAME}" ]]; then
    echo " ! Missing required env var JENV_NAME"
    exit 1
  fi
  if [[ "${JENV_NAME^^}" != *MAAS* ]]; then
    f_line "Deleting undercloud tenant's nova sec groups for $JENV_NAME"
    for i in $(nova secgroup-list | egrep "juju-osci-sv|juju-osci-lb|juju-osci-tk" | grep "$JENV_NAME" | awk '{ print $2 }'); do echo $i; nova secgroup-delete $i; done
  fi
}


function f_nice_finish() {
# Generally run at the end of every build
  if [[ "${NO_POST_DESTROY^^}" != "TRUE" ]]; then
    f_tmp_cleanup
  fi
  if [[ "${NO_POST_DESTROY^^}" != "TRUE" ]] && [[ "${SKIP_JUJU_LXC^^}" != "TRUE" ]]; then
    f_line "Destroy"
    ${OSCI_ROOT}/job-parts/juju1_bighammer_destroy.sh YES
    f_delete_neutron_ext_net_ports

    f_line "Post-Destroy Info Dump"
    export SUBNET_ID=$(neutron net-show $(cat $JENKINS_HOME/.juju/environments.yaml | grep network: | head -n1 | awk '{ print $2 }') | grep subnets | awk '{ print $4 }')
    set -x
    neutron port-list | grep "$SUBNET_ID" 2>&1 | tee $WORKSPACE/post-destroy-ports.txt
    neutron net-list | grep "$SUBNET_ID" 2>&1 | tee $WORKSPACE/post-destroy-networks.txt
    neutron subnet-list | grep "$SUBNET_ID" 2>&1 | tee $WORKSPACE/post-destroy-subnets.txt
    nova secgroup-list | grep "$JENV_NAME" 2>&1 | tee $WORKSPACE/post-destroy-secgroups.txt
    nova list | grep "$JENV_NAME" 2>&1 | tee $WORKSPACE/post-destroy-instances.txt
    set +x

    f_line "Post-Destroy Additional Clean-up"
    f_delete_secgroups ||:
    f_mojo_broom
    f_osi_broom
  fi
}


function f_check_fail() {
# Check various indicators of test failure and force job failure if found.

  # Check bundletester report file for failure indicator
  if [[ -f ${BUILT_ASSET_DIR-$CO_DIR}/func-results.json ]]; then
    func_nonzero_retcodes=$(grep "\"\?returncode\"\?:" func-results.json | awk '{ print $2 }' | tr -d ',' | grep -v "0" ||:)
    # Ex. sample data from func-results.json:
    #      "returncode": 0,
    #      "returncode": 4,
    #      "returncode": 1,
    # Produces non-zero return codes:
    # 1
    # 4
    if [[ -n "$func_nonzero_retcodes" ]]; then
      echo " ! Bundletester report indicates one or more tests failed (ret code(s): $func_nonzero_retcodes)"
      echo " ! Exiting 1."
      exit 1
    else
      echo " . Bundletester report indicates no test failures (OK)"
    fi
  fi

  # Check for failure touchfiles and fail if found
  lso="$(ls $WORKSPACE -1)"
  if [[ "$lso" == *.failed* ]] || \
     [[ "$lso" == *.missing* ]] || \
     [[ "$lso" == *.bad* ]]; then
    echo " ! Exiting 1."
    exit 1
  fi
}


function f_post_mp_comment() {
# Post mp comment if triggered by a merge proposal
  if [[ -n "$MP_TRIGGER" ]]; then
    cd $OSCI_ROOT
    tools/mp_comment.py ||:
  fi
}


function f_set_build_descr() {
# Collect the FYI files for build description
  cd $WORKSPACE
  b_d="$(ls -1 fyi*.* 2> /dev/null | cut -d '.' -f 2,3 | tr '\n' ' ' ||:)"

  if [[ -n "$b_d" ]]; then
    # Output build description for jenkins plugin pickup by regex
    echo "_:${b_d}"
  fi
}


function f_get_func_results_file() {
# Copy functional test results file if it exists
  cp -fv ${BUILT_ASSET_DIR-$CO_DIR}/func-results.json $WORKSPACE/func-results.json.txt 2>/dev/null ||:
}


function f_set_co_dir() {
  mkdir -p $JENKINS_HOME/checkout/$EXECUTOR_NUMBER
  export CO_DIR=$JENKINS_HOME/checkout/$EXECUTOR_NUMBER/$BASE_NAME
}

function f_check_out_lp_branch() {
# check out branch
  if [[ -z "$BASE_NAME" ]] || [[ -z "$BRANCH" ]]; then
    echo " ! No BRANCH and/or BASE_NAME env vars."
    touch $WORKSPACE/fyi-${test:0:4}.env-vars.bad
  else
    mkdir -pv $JENKINS_HOME/checkout
    f_set_co_dir
    MAKEFILE=$CO_DIR/Makefile

    if [[ -d $CO_DIR ]]; then
      echo " - Removing existing checkout @ $CO_DIR"
      rm -rf $CO_DIR
    fi
    echo " + Creating new checkout of $BRANCH @ $CO_DIR."
    bzr co --lightweight $BRANCH $CO_DIR
    bzr revno $CO_DIR 2>&1 | tee $WORKSPACE/branch-info.txt
    bzr info $CO_DIR 2>&1 | tee -a $WORKSPACE/branch-info.txt
    bzr log $CO_DIR -l1 2>&1 | tee -a $WORKSPACE/branch-info.txt
  fi

  if [[ "${CHARMHELPER_SYNC^^}" == "TRUE" ]]; then
    # sync charmhelpers if boolean is set
    make sync
  fi
}


function f_virtualenv_legacy() {
# Make the virtualenv target, if it exists, and wrapped in proxy
# environment variables, if they exist.  This is for charms which
# use legacy (non-tox) virtual envs..
  f_proxy_if_proxy
  make virtualenv ||:
  make venv ||:
  make .venv ||:
  make .venv2 ||:
  make .venv3 ||:
  f_proxy_unset
}


function f_run_makefile_test() {
# Run makefile-based test (generally lint, unit or amulet)
  cd ${BUILT_ASSET_DIR-$CO_DIR}

  # Always attempt legacy virtual env creation method for legacy charms
  f_virtualenv_legacy

  # Blacklist all deploy-from-source amulet tests from automation if they exist
  rm -fv tests/0*git ||:
  rm -fv tests/dfs* ||:

  # Forcefully blacklist EOL combos if they exist
  rm -fv tests/*juno* ||:
  rm -fv tests/*utopic* ||:
  rm -fv tests/*vivid* ||:
  rm -fv tests/*wily* ||:

  # sstring array set by job
  q_sstring_l=( "${sstring[@]/#/\"}" )
  q_sstring=( "${q_sstring_l[@]/%/\"}" )
  echo " . Running test from Makefile."
  cmd='${OSCI_ROOT}/tools/make_exec.py Makefile '"${q_sstring[*]}"' |& tee $WORKSPACE/${test}-test.txt; ret_code=${PIPESTATUS[0]}'
  echo $cmd
  eval $cmd
  echo "Returned:  ${ret_code}"
  case "$ret_code" in
    3)
      echo " ! No Makefile found."
      touch $WORKSPACE/fyi-${test:0:4}.makefile.missing
      ;;
    2)
      echo " ! ${test^} test not detected in Makefile."
      touch $WORKSPACE/fyi-${test:0:4}.${test}-test.missing
      ;;
    0)
      echo " . ${test^} test returned 0."
      ;;
    *)
      echo " ! ${test^} test failed."
      touch $WORKSPACE/fyi-${test:0:4}.${test}-test.failed
      ;;
  esac
}


function f_run_make_or_tox_test() {
# Inspect Makefile and tox.ini for matches and run the first match.
  cd ${BUILT_ASSET_DIR-$CO_DIR}

  # Sniff tox.ini and Makefile
  unset OSCI_MAKE_TARGET_FOUND
  unset OSCI_TOX_TARGET_FOUND
  for t in "${sstring[@]}"; do
    if ${OSCI_ROOT}/tools/make_exec.py Makefile "$t" -X &> /dev/null; then
      echo " . Make target containing (\"$t\") exists"
      export OSCI_MAKE_TARGET_FOUND="True"
      break
    fi
  done
  for t in "${tox_e_try[@]}"; do
    if grep "^\[testenv:${t}\]" tox.ini &> /dev/null; then
      echo " . Tox target (\"$t\") exists"
      export OSCI_TOX_TARGET_FOUND="$t"
      break
    fi
  done

  # Execute target
  if [[ -n "$OSCI_MAKE_TARGET_FOUND" ]]; then
    echo " . Calling tests via Makefile"
    f_run_makefile_test
  elif [[ -n "$OSCI_TOX_TARGET_FOUND" ]]; then
    echo " . Executing tox environment: $t"
    f_run_tox_environment $t
  else
    echo " ! Unable to determine make or tox targets to run"
    echo "Tried to find make targets containing one of: ${sstring[*]}"
    echo "Tried to find tox a environment named: ${tox_e_try[*]}"
    touch $WORKSPACE/makefile-or-toxini.missing
  fi
}


function f_run_tox_environment() {
# Run tox with optional environment (target) as the parameter
  if [[ -n "$1" ]]; then
    PARAM1="-e $1"
  else
    PARAM1=""
  fi

  f_line "Executing tox $PARAM1"
  tox $PARAM1 2>&1 | tee $WORKSPACE/tox-output.txt; ret_code=${PIPESTATUS[0]}
  echo "Tox returned:  ${ret_code}"
  if [[ "${ret_code}" != "0" ]]; then
      touch $WORKSPACE/tox.failed
  fi
}


function f_charm_proof_examine() {
# Examine test output, fail if charm proof ERR or WARN found.
  proof_errors="$(egrep 'W:|E:' ${WORKSPACE}/${test}-test.txt ||:)"
  if [[ -n "$proof_errors" ]]; then
      echo " ! Charm proof failed."
      touch $WORKSPACE/fyi-${test:0:4}.charm-proof.failed
  fi
}


function f_collect_juju_logs() {
# Collect juju stat and logs
  echo "Collecting juju status and unit logs..."

  # Check again before collecting logs
  #f_juju_status_health_check "-collect" ||:

  f_juju_stat "-collect" ||:

  . $JENKINS_HOME/novarc
  ${OSCI_ROOT}/tools/nova_instances_summarize.py ||:
  f_collect_tool_bzr_revnos ||:
  ${OSCI_ROOT}/tools/juju_pull_logs.py ||:
}


function f_juju_stat() {
  juju stat --format=yaml &> $WORKSPACE/juju-stat-yaml${1}.txt ||:
  juju stat --format=tabular &> $WORKSPACE/juju-stat-tabular${1}.txt ||:
}


function f_juju_bootstrap() {
  f_line "Bootstrapping juju with client: $(juju version), model: $(juju switch)"
  echo " . Bootstrap parameters: $1 $2 $3 $4 $5"
  juju bootstrap $1 $2 $3 $4 $5 2>&1 | tee $WORKSPACE/juju-bootstrap.txt
  if [[ "${PIPESTATUS[0]}" != "0" ]]; then
    echo " ! Bootstrap failed."
    touch $WORKSPACE/fyi-${test:0:4}.${test}-bootstrap.failed
  fi
}


function f_proxy_if_proxy() {
# Set http proxy if already set for amulet
  if [[ -n "$AMULET_HTTP_PROXY" ]]; then
    export NO_PROXY=$OSCI_NO_PROXY
    export HTTP_PROXY=$AMULET_HTTP_PROXY
    export HTTPS_PROXY=$(echo $AMULET_HTTP_PROXY | sed 's/http/https/g')
    export http_proxy=$HTTP_PROXY
    export https_proxy=$HTTPS_PROXY
    echo " + Enabling HTTP(S) proxies."
  fi
}


function f_proxy_unset() {
  echo " - Unsetting HTTP(S) proxies if set."
  unset NO_PROXY
  unset HTTP_PROXY
  unset HTTPS_PROXY
  unset no_proxy
  unset http_proxy
  unset https_proxy
}


function f_amulet_no_teardown() {
# Hack up the makefile to prevent amulet from tearing down enviro on failured
  cd $CO_DIR
  $OSCI_ROOT/tools/find_replace.py -yd -i Makefile -o Makefile -f "juju test -v" -r "juju test --set-e -v" ||:
}


function f_amulet_set_timeout() {
# Override known common timeout values in legacy amulet test runners
  $OSCI_ROOT/tools/find_replace.py -yd -i Makefile -o Makefile -f "--timeout 900" -r "--timeout 2700" ||:
  $OSCI_ROOT/tools/find_replace.py -yd -i Makefile -o Makefile -f "--timeout 1200" -r "--timeout 2700" ||:
}


# DEPRECATED (applies only to deb-installed amulet)
#function f_amulet_patch_timeout_seconds_announce() {
# A hack patch for amulet/helper.py to include the timeout value in the error output
#  f_line "amulet patch to announce timeout value upon error"
#  cd /usr/lib/python2.7/dist-packages/amulet
#  sudo patch --verbose -Nb helpers.py < ${OSCI_ROOT}/dev/patches/amulet-timeout-seconds-announce.patch ||:
#}
#function f_amulet_patch_no_symbols() {
# A hack patch for charmtools/test.py to not use symbols
#  f_line "amulet (charmtools) patch to not use symbols"
#  cd /usr/lib/python2.7/dist-packages/charmtools
#  sudo patch -Nb test.py < ${OSCI_ROOT}/dev/patches/amulet-no-symbols.patch ||:
#}
#function f_charmhelpers_amulet_timeout_bump() {
# A hack patch for tests/charmhelpers/contrib/amulet/deployment.py
# to override the existing hard-coded timeout values.
#  f_line "charmhelpers amulet deploy patch to override hard-coded timeout value"
#  cd ${BUILT_ASSET_DIR-$CO_DIR}/tests/charmhelpers/contrib/amulet ||:
#  sudo patch -Nb deployment.py < ${OSCI_ROOT}/dev/patches/charmhelpers-amulet-timeout-bump.patch ||:
#  f_line "end patch"
#}
#function f_amulet_patch_destroy_force_and_sleep() {
# A hack patch for charmtools/test.py to use --force on destroy,
# and to sleep after destroy, re: https://bugs.launchpad.net/juju-core/1.24/+bug/1335885
#  f_line "amulet patch to force destroy then sleep (https://bugs.launchpad.net/juju-core/1.24/+bug/1335885)"
#  sudo patch -Nb /usr/lib/python2.7/dist-packages/charmtools/test.py < ${OSCI_ROOT}/dev/patches/amulet-destroy-force-and-sleep.patch ||:
#  f_line "end patch"
#}


function f_collect_tool_bzr_revnos() {
# Collect charm branch revno info from O-C-T path
  f_line "Collecting tool bzr revno info for select tools"

  if [[ -n "$CO_DIR" ]] && [[ -f "${CO_DIR}/.bzr/branch/branch.conf" ]]; then
    revno="$(bzr revno ${CO_DIR} ||:)"
    echo "${CO_DIR}: ${revno}" | tee -a $WORKSPACE/revnos.txt
    bzr info ${CO_DIR} | egrep 'root|branch|checkout' | tee -a $WORKSPACE/revnos.txt
    echo -e "\n" | tee -a $WORKSPACE/revnos.txt
  else
    echo " ! Skipping ${CO_DIR}, does not appear to be a bzr branch."
  fi

  if [[ -d "${OPENSTACK_CHARM_TESTING_ROOT}/${UBUNTU_RELEASE}" ]]; then
    revno="$(bzr revno ${OPENSTACK_CHARM_TESTING_ROOT} ||:)"
    echo "${OPENSTACK_CHARM_TESTING_ROOT}: ${revno}" | tee -a $WORKSPACE/revnos.txt
    bzr info ${OPENSTACK_CHARM_TESTING_ROOT} | egrep 'root|branch|checkout' | tee -a $WORKSPACE/revnos.txt
    echo -e "\n" | tee -a $WORKSPACE/revnos.txt

    for c in $(ls ${OPENSTACK_CHARM_TESTING_ROOT}/${UBUNTU_RELEASE} -1); do
      revno="$(bzr revno ${OPENSTACK_CHARM_TESTING_ROOT}/${UBUNTU_RELEASE}/${c} ||:)"
      echo "${c}: ${revno}" 2>&1 | tee -a $WORKSPACE/revnos.txt
      bzr info ${OPENSTACK_CHARM_TESTING_ROOT}/${UBUNTU_RELEASE}/${c} | egrep 'checkout of branch' | tee -a $WORKSPACE/revnos.txt
    done
  else
    echo " ! Skipping o-c-t/series/charm revno inspection, does not exist: ${OPENSTACK_CHARM_TESTING_ROOT}/${UBUNTU_RELEASE}"
  fi
}

function f_gather_package_info() {
  echo " . Gathering package info"
  tmp_file="$(mktemp)"
  cat > "$tmp_file" << "--EOF"
  amulet
  charm
  charm-tools
  juju
  juju-1.25
  juju-2.0
  juju-core
  juju-deployer
  lxc
  lxd
  mojo
  python-amqp
  python-ceilometerclient
  python-cinderclient
  python-glanceclient
  python-heatclient
  python-jujuclient
  python-keystoneclient
  python-neutronclient
  python-novaclient
  python-openstackclient
  python-swiftclient
  python-tox
  linux-virtual
--EOF

  for line in $(cat $tmp_file); do
    str="${str}${line}|"
    apt-cache policy $line 2>&1 | tee -a $WORKSPACE/pkg_info.txt
  done
  match=$(echo $str | head -c -2)
  eval "dpkg -l | egrep '$match'" 2>&1 | tee -a $WORKSPACE/pkg_info.txt
  rm -vf $tmp_file
}


function f_get_env_files() {
# Get juju instance files
  export env_files="$(ls -1 $j_env_dir/*env 2> /dev/null ||:)"
}


function f_get_my_nova_instances() {
# Get nova instances for the current juju environment
  if [[ "${JENV_NAME^^}" != *MAAS* ]]; then
    export my_instances=$(timeout 60 nova list | grep "juju\-${JENV_NAME}\-machine" ||:)
  fi
}


function f_tmp_cleanup(){
# Clean up tmp dir items as the jenkins user
  f_line "Cleaning up /tmp"
  echo " . Safe to ignore file/dir permission errors here."
  find /tmp -type f -user jenkins -writable -readable -mmin +60 | xargs rm -rf ||:
}


function f_line() {
# Useful to announce in the console log
  echo "====>  ${1}  <======================="
}


# Go.
f_openstack_uos_env_var_normalize

if [[ "${SKIP_JUJU_LXC^^}" != "TRUE" ]]; then
  # Default to Juju previous major version
  f_go_juju1
  f_juju1_get_model_name
  f_juju1_model_switcher
fi
